//============================================================================
// 635 WinTest Code.
// Copyright 2001, Crystalfontz America, Inc. Written by Brent A. Crosby
// www.crystalfontz.com, brent@crystalfontz.com
//============================================================================
#include "stdafx.h"
#include "typedefs.h"
#include "serial.h"
//------------------------------------------------------------------------------
const char
	*serial_port_names[9]=
   {"COM1","COM2","COM3","COM4","COM5","COM6","COM7","COM8","COM9"};
//------------------------------------------------------------------------------
HANDLE
  hCom=NULL;
/*---------------------------------------------------------------------------*/
//This is some code that kind of makes the windows stuff look a little
//like the DOS/633 interrupt driven serial stuff. Basically there is a
//circular buffer, and Sync_Read_Buffer() uses ReadFile() to put data
//into the circular buffer much like the DOS stuff uses an ISR to put
//the data into the buffer. Then the rest of the functions work like
//the counterparts in the 633,
#define RECEIVEBUFFERSIZE    4096
ubyte
  SerialReceiveBuffer[RECEIVEBUFFERSIZE];
DWORD
  ReceiveBufferHead;
DWORD
  ReceiveBufferTail;
DWORD
  ReceiveBufferTailPeek;
//------------------------------------------------------------------------------
int Serial_Init(int port, int baud_rate)
	{
	DCB
		dcb;
	BOOL
		fSuccess;

	if(hCom)
		CloseHandle(hCom);

	hCom = CreateFile(
		serial_port_names[port],
		GENERIC_READ | GENERIC_WRITE,
		0,             /* comm devices must be opened w/exclusive-access */
		NULL,          /* no security attrs */
		OPEN_EXISTING, /* comm devices must use OPEN_EXISTING */
		0,             /* not overlapped I/O */
		NULL           /* hTemplate must be NULL for comm devices */
		);

	if(hCom == INVALID_HANDLE_VALUE)
    {
    hCom=0;
		return(1);
    }

	/*
	 * Omit the call to SetupComm to use the default queue sizes.
	 * Get the current configuration.
	 */
	fSuccess = GetCommState(hCom, &dcb);
	if(!fSuccess)
    {
    hCom=0;
		return(2);
    }

  int
    baud_rates[8]=
      {1200,2400,4800,9600,19200,38400,57600,115200};

	/* Fill in the DCB: baud=9600, 8 data bits, no parity, 1 stop bit. */
	dcb.BaudRate = baud_rates[baud_rate];
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.fRtsControl = RTS_CONTROL_ENABLE;  //customary, but not necessary
	dcb.fDtrControl = DTR_CONTROL_ENABLE;  //customary, but not necessary
	dcb.fAbortOnError = FALSE; 
	fSuccess = SetCommState(hCom, &dcb);

	if(!fSuccess)
    {
    hCom=0;
		return(3);
    }

	COMMTIMEOUTS
		our_timeouts;

	if(TRUE!=GetCommTimeouts(
			hCom,						// handle of communications device
			&our_timeouts))		// address of comm. time-outs structure
    {
    hCom=0;
    return(4);
    }

	//9600 is 1 byte per mS. Set it to 2 mS so the read will stay
	//as long as data is coming in. Will wait at least 100mS for
	//the 633 to respond.
	our_timeouts.ReadIntervalTimeout=2000;
	our_timeouts.ReadTotalTimeoutMultiplier=2000;  // 1 second
	our_timeouts.ReadTotalTimeoutConstant=2000;

	if(TRUE!=SetCommTimeouts(
			hCom,			    	// handle of communications device
			&our_timeouts)) 	// address of comm. time-outs structure
    {
    hCom=0;
		return(5);
    }

  ReceiveBufferHead=ReceiveBufferTail=0;;

	return(0);
	}
//------------------------------------------------------------------------------
void Uninit_Serial()
  {
	CloseHandle(hCom);
	hCom=NULL;
  }
//------------------------------------------------------------------------------
void SendByte(unsigned char datum)
	{
	DWORD
		bytes_written;
	bytes_written=0;

  if(hCom)
    {
		if(TRUE!=WriteFile(
				hCom,							// handle of file to write to
				&datum,      			// address of data to write to file
				1,              	// number of bytes to write
				&bytes_written,	  // address of number of bytes written
				NULL))	 		 			// addr. of structure needed for overlapped I/O
			{
			printf("Send_Character(): WriteFile() failed.");
			return;
			}
		if(bytes_written!=1)
			{
  		printf("Send_Character(): only %d of %d bytes sent.",
						bytes_written,1);
			return;
			}
    }	
	}
//------------------------------------------------------------------------------
//Gets incoming data from Window's ReadFile() and puts it into
//SerialReceiveBuffer[].
void Sync_Read_Buffer(ubyte expected_bytes)
  {
	ubyte
		Incoming[4096];
	DWORD
		BytesRead;

	COMSTAT
		status;
	DWORD
		errors;

  if(!hCom)
    return;

	//Find out how much data is available to be read.
	ClearCommError(
		hCom,      // handle to communications device
		&errors,   // pointer to variable to receive error codes
		&status);  // pointer to buffer for communications status

	if(status.cbInQue)
		{
		if(!ReadFile(hCom,               // handle of file to read
					 			 (LPVOID *)Incoming, // pointer to buffer that receives data
								 status.cbInQue,     // number of bytes to read
								 &BytesRead,         // pointer to number of bytes read
								 NULL))              // Overlaped structure address
			{
			printf("Sync_Read_Buffer(): ReadFile() Error!");
			}

		//Read the incoming ubyte, store it,
		for(DWORD i=0;i<BytesRead;i++)
			{
			SerialReceiveBuffer[ReceiveBufferHead]=Incoming[i];
			//Increment the pointer and wrap if needed.
			ReceiveBufferHead++;
			if(RECEIVEBUFFERSIZE<=ReceiveBufferHead)
				ReceiveBufferHead=0;
			}
		}
  }
/*---------------------------------------------------------------------------*/
DWORD BytesAvail(void)
  {
  //LocalReceiveBufferHead and return_value are a signed variables.
  register int
    LocalReceiveBufferHead;
  register int
    return_value;
    
  //Get a register copy of the head pointer, since an interrupt that
  //modifies the head pointer could screw up the value. Convert it to
  //our signed local variable.
  LocalReceiveBufferHead=ReceiveBufferHead;
  if((return_value=(LocalReceiveBufferHead-(int)ReceiveBufferTail))<0)
    return_value+=RECEIVEBUFFERSIZE;
  return(return_value);
  }
/*---------------------------------------------------------------------------*/
ubyte GetByte(void)
  {
  register DWORD
    LocalReceiveBufferTail;
  register DWORD
    LocalReceiveBufferHead;
  register ubyte
    return_byte;

  //Get a register copy of the tail pointer for speed and size.
  LocalReceiveBufferTail=ReceiveBufferTail;
  
  //Get a register copy of the head pointer, since an interrupt that
  //modifies the head pointer could screw up the value.
  LocalReceiveBufferHead=ReceiveBufferHead;
  
  //See if there are any more bytes available.
  if(LocalReceiveBufferTail!=LocalReceiveBufferHead)
    {
    //There is at least one more ubyte.
    return_byte=SerialReceiveBuffer[LocalReceiveBufferTail];
    //Increment the pointer and wrap if needed.
    LocalReceiveBufferTail++;
    if(RECEIVEBUFFERSIZE<=LocalReceiveBufferTail)
      LocalReceiveBufferTail=0;
    //Update the real ReceiveBufferHead with our register copy. Make sure
    //the ISR does not get serviced during the transfer.
    ReceiveBufferTail=LocalReceiveBufferTail;
    }
  return(return_byte);
  }
/*---------------------------------------------------------------------------*/
DWORD PeekBytesAvail(void)
  {
  //LocalReceiveBufferHead and return_value are a signed variables.
  register int
    LocalReceiveBufferHead;
  register int
    return_value;
    
  //Get a register copy of the head pointer, since an interrupt that
  //modifies the head pointer could screw up the value. Convert it to
  //our signed local variable.
  LocalReceiveBufferHead=ReceiveBufferHead;
  if((return_value=(LocalReceiveBufferHead-(int)ReceiveBufferTailPeek))<0)
    return_value+=RECEIVEBUFFERSIZE;
  return(return_value);
  }
/*---------------------------------------------------------------------------*/
void Sync_Peek_Pointer(void)
  {
  ReceiveBufferTailPeek=ReceiveBufferTail;
  }
/*---------------------------------------------------------------------------*/
void AcceptPeekedData(void)
  {
  ReceiveBufferTail=ReceiveBufferTailPeek;
  }
/*---------------------------------------------------------------------------*/
ubyte PeekByte(void)
  {
  register int
    LocalReceiveBufferTailPeek;
  register int
    LocalReceiveBufferHead;
  register ubyte
    return_byte;

  //Get a register copy of the tail pointer for speed and size.
  LocalReceiveBufferTailPeek=ReceiveBufferTailPeek;
  
  //Get a register copy of the head pointer, since an interrupt that
  //modifies the head pointer could screw up the value.
  LocalReceiveBufferHead=ReceiveBufferHead;
  
  //See if there are any more bytes available.
  if(LocalReceiveBufferTailPeek!=LocalReceiveBufferHead)
    {
    //There is at least one more ubyte.
    return_byte=SerialReceiveBuffer[LocalReceiveBufferTailPeek];
    //Increment the pointer and wrap if needed.
    LocalReceiveBufferTailPeek++;
    if(RECEIVEBUFFERSIZE<=LocalReceiveBufferTailPeek)
      LocalReceiveBufferTailPeek=0;
    //Update the real ReceiveBufferHead with our register copy. Make sure
    //the ISR does not get serviced during the transfer.
    ReceiveBufferTailPeek=LocalReceiveBufferTailPeek;
    }
  return(return_byte);
  }
/*---------------------------------------------------------------------------*/ 
